/****************************************************  Product Functions  *****************************************************

	Red Rapids
	797 North Grove Rd
	Suite 101
	Richardson, TX 75081
	www.redrapids.com | support@redrapids.com

	The following functions are used to initialize the hardware specific to each SigFPGA model.

	Rev History: 02-18-2014:	Added Model 372 and Model 373 support.
				 12-08-2014:	Added Model 371 support.
				 10-20-2014:	Inital release.

	Copyright 2014, Red Rapids, Inc.

****************************************************  Product Functions  *****************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "adapter.h"
#include "sigfpga.h"
#include "dma_functions.h"
#include "common_functions.h"
#include "channel_functions.h"
#include "device_functions.h"
#include "product_functions.h"


// Intitialize the Model 371 hardware.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long M371Initialize(s_Adapter *p_HW, s_M371Devices M371Devices, s_Channel *p_RXChannel)
{
	unsigned long Status = 0;
	size_t cnt = 0;
	s_User *User = NULL;

	// Cast the user defined pointer from the hardware handle.
	User = (s_User *)(p_HW->User);

	// The Model 371 is equipped with two receiver channels and no transmitter channels.
	User->RXChannelCount = ChannelCount(p_HW, 0);
	User->TXChannelCount = ChannelCount(p_HW, 1);
	// The Model 371 RX channels produce a 16-bit data item size.
	User->RXDataItemSize = 16;
	User->TXDataItemSize = 0;

	// Configure the I/O signal direction and impedance of the command/status port expander.
	// Note: The 100 MHz configuration flash clock is disabled.
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x3, 0xF8);
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x7, 0x07);
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x5, 0x00);

	// Configure the dual ADC that is assigned to RX channels #1 and #2 by loading the default values.
	for (cnt = 0; cnt < 21; cnt++)
		Status += ADS42LB69Write(p_HW, M371_RX1TO2_ADC, M371Devices.RX1TO2_ADC.Payload[cnt][0], M371Devices.RX1TO2_ADC.Payload[cnt][1]);

	// Configure the trim DACs from the default structure values.
	// Note: Trim DACs are only present on DC coupled units.
	for (cnt = 0; cnt < 3; cnt++)
	{
		Status += LTC1661Write(p_HW, M371_RX1_TRIM_DAC, M371Devices.RX1_TRIM_DAC.Payload[cnt][0], M371Devices.RX1_TRIM_DAC.Payload[cnt][1]);
		Status += LTC1661Write(p_HW, M371_RX2_TRIM_DAC, M371Devices.RX2_TRIM_DAC.Payload[cnt][0], M371Devices.RX2_TRIM_DAC.Payload[cnt][1]);
	}

	// Configure the amplifier enables from the default structure values.
	// Note: Amplifiers are only present on DC coupled units.
	for (cnt = 0; cnt < 2; cnt++)
		Status += ADL5566Write(p_HW, CMD_STAT1, M371Devices.RX1TO2_AMP_ENBL.Enable[cnt][0], M371Devices.RX1TO2_AMP_ENBL.Enable[cnt][1]);

	// Determine whether the external sample clock is active.
	if (ClockStatus(p_HW, CMD_STAT1) < 2)
		// An external clock was detected, so change the selected input to the clock distribution chip.
		// Note: The default input is the on-board synthesizer.
		M371Devices.ClockDist.Payload[9][1] = 0x02;

	// Hardware reset the clock distribution chip.
	Status += AD9512Write(p_HW, M371_CLK_DIST, 0x0000, 0x30);
	Status += AD9512Write(p_HW, M371_CLK_DIST, 0x005A, 0x01);
	Status += AD9512Write(p_HW, M371_CLK_DIST, 0x0000, 0x10);
	Status += AD9512Write(p_HW, M371_CLK_DIST, 0x005A, 0x01);
	// Configure the clock distribution chip by loading the default values.
	for (cnt = 0; cnt < 22; cnt++)
		Status += AD9512Write(p_HW, M371_CLK_DIST, M371Devices.ClockDist.Payload[cnt][0], M371Devices.ClockDist.Payload[cnt][1]);

	return(Status);
}

// Intitialize the Model 372 hardware.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long M372Initialize(s_Adapter *p_HW, s_M372Devices M372Devices, s_Channel *p_RXChannel)
{
	unsigned long Status = 0;
	size_t cnt = 0;
	s_User *User = NULL;

	// Cast the user defined pointer from the hardware handle.
	User = (s_User *)(p_HW->User);

	// Query the hardware for the number of receiver and transmitter channels.
	User->RXChannelCount = ChannelCount(p_HW, 0);
	User->TXChannelCount = ChannelCount(p_HW, 1);
	// The Model 372 RX and TX channels process a 16-bit data item size.
	User->RXDataItemSize = 16;
	User->TXDataItemSize = 16;

	// Configure the I/O signal direction and impedance of the command/status port expander.
	// Note: The 100 MHz configuration flash clock is disabled.
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x3, 0xF8);
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x7, 0x07);
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x5, 0x00);

	// Reset the clock distribution chip.
	Status += LMK01000Write(p_HW, M372_CLK_DIST, 0x0, 0x8000010);
	// Determine whether the external sample clock is active.
	if (ClockStatus(p_HW, CMD_STAT1) < 2)
		// An external clock was detected, so change the clock distribution chip input to CLKin0.
		// Note: The default input is the on-board synthesizer (CLKin1).
		M372Devices.ClockDist.Payload[9][1] &= 0xDFFFFFF;
	// Configure the clock distribution chip by loading the modified default values.
	for (cnt = 0; cnt < 10; cnt++)
		Status += LMK01000Write(p_HW, M372_CLK_DIST, M372Devices.ClockDist.Payload[cnt][0], M372Devices.ClockDist.Payload[cnt][1]);
	// Synchronize the outputs of the clock distribution chip.
	Status += LMK01000Sync(p_HW, M372_CLK_DIST_SYNC);

	// Reset the dual ADC that is assigned to RX channels #1 and #2.
	Status += AD9652Write(p_HW, M372_RX1TO2_ADC, 0x0, 0x3C);
	// Wait 500 us for the SPI interface to become available after reset.
	MicrosecondTimer(500);
	// Configure the dual ADC that is assigned to RX channels #1 and #2 by loading the default values.
	for (cnt = 0; cnt < 26; cnt++)
		Status += AD9652Write(p_HW, M372_RX1TO2_ADC, M372Devices.RX1TO2_ADC.Payload[cnt][0], M372Devices.RX1TO2_ADC.Payload[cnt][1]);

	// Configure the RX trim DACs from the default structure values.
	// Note: Trim DACs are only present on RX DC coupled units.
	for (cnt = 0; cnt < 3; cnt++)
	{
		Status += LTC1661Write(p_HW, M372_RX1_TRIM_DAC, M372Devices.RX1_TRIM_DAC.Payload[cnt][0], M372Devices.RX1_TRIM_DAC.Payload[cnt][1]);
		Status += LTC1661Write(p_HW, M372_RX2_TRIM_DAC, M372Devices.RX2_TRIM_DAC.Payload[cnt][0], M372Devices.RX2_TRIM_DAC.Payload[cnt][1]);
	}

	// Configure the RX amplifier enables from the default structure values.
	// Note: Amplifiers are only present on RX DC coupled units.
	Status += ADA9247Write(p_HW, CMD_STAT1, M372Devices.RX1_AMP_ENBL.Enable[0], M372Devices.RX1_AMP_ENBL.Enable[1]);
	Status += ADA9247Write(p_HW, CMD_STAT1, M372Devices.RX2_AMP_ENBL.Enable[0], M372Devices.RX2_AMP_ENBL.Enable[1]);

	// Reset the dual DAC that is assigned to TX channels #3 and #4.
	Status += AD9142AWrite(p_HW, M372_TX3TO4_DAC,0x00, 0x20);
	// Configure the dual DAC that is assigned to TX channels #3 and #4 by loading the default values.
	for (cnt = 0; cnt < 63; cnt++)
		Status += AD9142AWrite(p_HW, M372_TX3TO4_DAC, M372Devices.TX3TO4_DAC.Payload[cnt][0], M372Devices.TX3TO4_DAC.Payload[cnt][1]);
	// Enable the DAC PLL automatic VCO band select feature.
	Status += AD9142AWrite(p_HW, M372_TX3TO4_DAC, 0x12, 0xC0);
	Status += AD9142AWrite(p_HW, M372_TX3TO4_DAC, 0x12, 0x80);
	// Wait 10 ms for the band select to complete.
	MicrosecondTimer(10000);
	// Reset the DAC FIFO.
	Status += AD9142AWrite(p_HW, M372_TX3TO4_DAC,0x25, 0x01);
	// Check the DAC FIFO status.
	if (!(AD9142ARead(p_HW, M372_TX3TO4_DAC, 0x25) & 0x02)) Status++;
	// Check the DAC DLL lock status.
	if (!(AD9142ARead(p_HW, M372_TX3TO4_DAC, 0x0E) & 0x80)) Status++;
	// Check the DAC PLL lock status.
	if (!(AD9142ARead(p_HW, M372_TX3TO4_DAC, 0x16) & 0x80)) Status++;
	// Power up the DAC outputs.
	Status += AD9142AWrite(p_HW, M372_TX3TO4_DAC, 0x01, 0x00);

	// Configure the TX trim DACs from the default structure values.
	// Note: Trim DACs are only present on TX DC coupled units.
	for (cnt = 0; cnt < 3; cnt++)
	{
		Status += LTC1661Write(p_HW, M372_TX3_TRIM_DAC, M372Devices.TX3_TRIM_DAC.Payload[cnt][0], M372Devices.TX3_TRIM_DAC.Payload[cnt][1]);
		Status += LTC1661Write(p_HW, M372_TX4_TRIM_DAC, M372Devices.TX4_TRIM_DAC.Payload[cnt][0], M372Devices.TX4_TRIM_DAC.Payload[cnt][1]);
	}

	// Configure the TX amplifier enables from the default structure values.
	// Note: Amplifiers are only present on TX DC coupled units.
	Status += ADA9247Write(p_HW, CMD_STAT1, M372Devices.TX3_AMP_ENBL.Enable[0], M372Devices.TX3_AMP_ENBL.Enable[1]);
	Status += ADA9247Write(p_HW, CMD_STAT1, M372Devices.TX4_AMP_ENBL.Enable[0], M372Devices.TX4_AMP_ENBL.Enable[1]);

	return(Status);
}

// Intitialize the Model 373 hardware.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long M373Initialize(s_Adapter *p_HW, s_M373Devices M373Devices, s_Channel *p_RXChannel)
{
	unsigned long Status = 0;
	size_t cnt = 0;
	s_User *User = NULL;

	// Cast the user defined pointer from the hardware handle.
	User = (s_User *)(p_HW->User);

	// Query the hardware for the number of receiver and transmitter channels.
	User->RXChannelCount = ChannelCount(p_HW, 0);
	User->TXChannelCount = ChannelCount(p_HW, 1);
	// The Model 373 RX channels process a 16-bit data item size.
	User->RXDataItemSize = 16;
	User->TXDataItemSize = 0;

	// Configure the I/O signal direction and impedance of the command/status port expander.
	// Note: The 100 MHz configuration flash clock is disabled.
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x3, 0xF8);
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x7, 0x07);
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x5, 0x00);

	// Reset the clock distribution chip.
	Status += LMK01000Write(p_HW, M373_CLK_DIST, 0x0, 0x8000010);
	// Determine whether the external sample clock is active.
	if (ClockStatus(p_HW, CMD_STAT1) < 2)
		// An external clock was detected, so change the clock distribution chip input to CLKin0.
		// Note: The default input is the on-board synthesizer (CLKin1).
		M373Devices.ClockDist.Payload[9][1] &= 0xDFFFFFF;
	// Configure the clock distribution chip by loading the modified default values.
	for (cnt = 0; cnt < 10; cnt++)
		Status += LMK01000Write(p_HW, M373_CLK_DIST, M373Devices.ClockDist.Payload[cnt][0], M373Devices.ClockDist.Payload[cnt][1]);
	// Synchronize the outputs of the clock distribution chip.
	Status += LMK01000Sync(p_HW, M373_CLK_DIST_SYNC);

	// Reset the dual ADC that is assigned to RX channels #1 and #2.
	Status += AD9652Write(p_HW, M373_RX1TO2_ADC, 0x0, 0x3C);
	// Wait 500 us for the SPI interface to become available after reset.
	MicrosecondTimer(500);
	// Configure the dual ADC that is assigned to RX channels #1 and #2 by loading the default values.
	for (cnt = 0; cnt < 26; cnt++)
		Status += AD9652Write(p_HW, M373_RX1TO2_ADC, M373Devices.RX1TO2_ADC.Payload[cnt][0], M373Devices.RX1TO2_ADC.Payload[cnt][1]);

	// Configure the RX trim DACs from the default structure values.
	// Note: Trim DACs are only present on RX DC coupled units.
	for (cnt = 0; cnt < 3; cnt++)
	{
		Status += LTC1661Write(p_HW, M373_RX1_TRIM_DAC, M373Devices.RX1_TRIM_DAC.Payload[cnt][0], M373Devices.RX1_TRIM_DAC.Payload[cnt][1]);
		Status += LTC1661Write(p_HW, M373_RX2_TRIM_DAC, M373Devices.RX2_TRIM_DAC.Payload[cnt][0], M373Devices.RX2_TRIM_DAC.Payload[cnt][1]);
	}

	// Configure the RX amplifier enables from the default structure values.
	// Note: Amplifiers are only present on RX DC coupled units.
	Status += ADA9247Write(p_HW, CMD_STAT1, M373Devices.RX1_AMP_ENBL.Enable[0], M373Devices.RX1_AMP_ENBL.Enable[1]);
	Status += ADA9247Write(p_HW, CMD_STAT1, M373Devices.RX2_AMP_ENBL.Enable[0], M373Devices.RX2_AMP_ENBL.Enable[1]);

	return(Status);
}

// Intitialize the Model 376 hardware.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long M376Initialize(s_Adapter *p_HW, s_M376Devices M376Devices, s_Channel *p_RXChannel)
{
	unsigned long Status = 0;
	size_t cnt = 0;
	s_User *User = NULL;

	// Cast the user defined pointer from the hardware handle.
	User = (s_User *)(p_HW->User);

	// The standard Model 376 has two receiver channels and no transmitter channels.
	User->RXChannelCount = ChannelCount(p_HW, 0);
	User->TXChannelCount = ChannelCount(p_HW, 1);
	// The SigFPGA hardware sign extends the 12-bit sample data to a 16-bit data item size.
	User->RXDataItemSize = 16;
	User->TXDataItemSize = 0;

	// Configure the I/O signal direction and impedance of the command/status port expander.
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x3, 0x38);
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x7, 0xC7);

	// Configure the ADC from the default structure values.
	for (cnt = 0; cnt < 11; cnt++)
		Status += ADC12D1600Write(p_HW, M376_RX1TO2_ADC, M376Devices.RX1TO2_ADC.Payload[cnt][0], M376Devices.RX1TO2_ADC.Payload[cnt][1]);

	// Configure the trim DACs from the default structure values.
	// Note: Trim DACs are only present on DC coupled units.
	for (cnt = 0; cnt < 3; cnt++)
	{
		Status += LTC1661Write(p_HW, M376_RX1_TRIM_DAC, M376Devices.RX1_TRIM_DAC.Payload[cnt][0], M376Devices.RX1_TRIM_DAC.Payload[cnt][1]);
		Status += LTC1661Write(p_HW, M376_RX2_TRIM_DAC, M376Devices.RX2_TRIM_DAC.Payload[cnt][0], M376Devices.RX2_TRIM_DAC.Payload[cnt][1]);
	}

	// Configure the amplifier enables from the default structure values.
	// Note: Amplifiers are only present on DC coupled units.
	for (cnt = 0; cnt < 2; cnt++)
		Status += ADL5566Write(p_HW, CMD_STAT1, M376Devices.RX1TO2_AMP_ENBL.Enable[cnt][0], M376Devices.RX1TO2_AMP_ENBL.Enable[cnt][1]);

	return(Status);
}

// Intitialize the Model 377 hardware.
// Return of zero indicates success, non-zero indicates an error condition.
unsigned long M377Initialize(s_Adapter *p_HW, s_M377Devices M377Devices, s_Channel *p_RXChannel)
{
	unsigned long Status = 0;
	size_t cnt = 0;
	s_User *User = NULL;

	// Cast the user defined pointer from the hardware handle.
	User = (s_User *)(p_HW->User);

	// The Model 377 can is equipped with four receiver channels and no transmitter channels.
	User->RXChannelCount = ChannelCount(p_HW, 0);
	User->TXChannelCount = ChannelCount(p_HW, 1);
	// The Model 377 RX channels produce a 16-bit data item size.
	User->RXDataItemSize = 16;
	User->TXDataItemSize = 0;

	// Configure the I/O signal direction and impedance of the command/status port expander.
	// Note: The 100 MHz configuration flash clock is disabled.
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x3, 0xF8);
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x7, 0x07);
	Status += FXL6408Write(p_HW, CMD_STAT1, 0x5, 0x00);

	// Configure the first ADC that is assigned to RX channels #1 and #2 by loading the default values.
	for (cnt = 0; cnt < 21; cnt++)
		Status += ADS42LB69Write(p_HW, M377_RX1TO2_ADC, M377Devices.RX1TO2_ADC.Payload[cnt][0], M377Devices.RX1TO2_ADC.Payload[cnt][1]);

	// Configure the second ADC that is assigned to RX channels #3 and #4 by loading the default values.
	for (cnt = 0; cnt < 21; cnt++)
		Status += ADS42LB69Write(p_HW, M377_RX3TO4_ADC, M377Devices.RX3TO4_ADC.Payload[cnt][0], M377Devices.RX3TO4_ADC.Payload[cnt][1]);

	// Configure the trim DACs from the default structure values.
	// Note: Trim DACs are only present on DC coupled units.
	for (cnt = 0; cnt < 3; cnt++)
	{
		Status += LTC1661Write(p_HW, M377_RX1_TRIM_DAC, M377Devices.RX1_TRIM_DAC.Payload[cnt][0], M377Devices.RX1_TRIM_DAC.Payload[cnt][1]);
		Status += LTC1661Write(p_HW, M377_RX2_TRIM_DAC, M377Devices.RX2_TRIM_DAC.Payload[cnt][0], M377Devices.RX2_TRIM_DAC.Payload[cnt][1]);
		Status += LTC1661Write(p_HW, M377_RX3_TRIM_DAC, M377Devices.RX3_TRIM_DAC.Payload[cnt][0], M377Devices.RX3_TRIM_DAC.Payload[cnt][1]);
		Status += LTC1661Write(p_HW, M377_RX4_TRIM_DAC, M377Devices.RX4_TRIM_DAC.Payload[cnt][0], M377Devices.RX4_TRIM_DAC.Payload[cnt][1]);
	}

	// Configure the amplifier enables from the default structure values.
	// Note: Amplifiers are only present on DC coupled units.
	for (cnt = 0; cnt < 2; cnt++)
	{
		Status += ADL5566Write(p_HW, CMD_STAT1, M377Devices.RX1TO2_AMP_ENBL.Enable[cnt][0], M377Devices.RX1TO2_AMP_ENBL.Enable[cnt][1]);
		Status += ADL5566Write(p_HW, CMD_STAT1, M377Devices.RX3TO4_AMP_ENBL.Enable[cnt][0], M377Devices.RX3TO4_AMP_ENBL.Enable[cnt][1]);
	}

	// Determine whether the external sample clock is active.
	if (ClockStatus(p_HW, CMD_STAT1) < 2)
		// An external clock was detected, so change the selected input to the clock distribution chip.
		// Note: The default input is the on-board synthesizer.
		M377Devices.ClockDist.Payload[9][1] = 0x02;

	// Hardware reset the clock distribution chip.
	Status += AD9512Write(p_HW, M377_CLK_DIST, 0x0000, 0x30);
	Status += AD9512Write(p_HW, M377_CLK_DIST, 0x005A, 0x01);
	Status += AD9512Write(p_HW, M377_CLK_DIST, 0x0000, 0x10);
	Status += AD9512Write(p_HW, M377_CLK_DIST, 0x005A, 0x01);
	// Configure the clock distribution chip by loading the default values.
	for (cnt = 0; cnt < 22; cnt++)
		Status += AD9512Write(p_HW, M377_CLK_DIST, M377Devices.ClockDist.Payload[cnt][0], M377Devices.ClockDist.Payload[cnt][1]);

	return(Status);
}
